<?php

declare(strict_types=1);

namespace Drupal\media_iframe_cookie\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Render\RendererInterface;
use Drupal\media_iframe_cookie\LazyBuilder\ExternalMediaRender;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Cookie;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller for external medias.
 */
class ExternalMediasController extends ControllerBase {

  /**
   * The external media render service.
   *
   * @var \Drupal\media_iframe_cookie\LazyBuilder\ExternalMediaRender
   */
  protected $externalMediaRender;

  /**
   * The request service.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructs a new ExternalMediasController object.
   *
   * @param \Drupal\media_iframe_cookie\LazyBuilder\ExternalMediaRender $externalMediaRender
   *   The external media render service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(ExternalMediaRender $externalMediaRender, RequestStack $requestStack, RendererInterface $renderer) {
    $this->externalMediaRender = $externalMediaRender;
    $this->request = $requestStack->getCurrentRequest();
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('media_iframe_cookie.external_media_render'),
      $container->get('request_stack'),
      $container->get('renderer')
    );
  }

  /**
   * Render external media.
   *
   * @param \Drupal\media\Entity\Media $media
   *   The media entity.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The ajax response.
   *
   * @throws \Exception
   */
  public function renderExternalMedia($media): AjaxResponse {

    // Check if ajax request.
    if (!$this->request->isXmlHttpRequest()) {
      throw new NotFoundHttpException();
    }

    // Init ajax response.
    $response = new AjaxResponse();

    // Get html selector.
    $selector = '.' . ExternalMediaRender::SELECTOR_ITEM . '_' . $media->id();

    // Get html content.
    $postRender = $this->externalMediaRender->getRenderExternalMedia(
      $media->get('field_media_external_medias')->value,
      (int) $media->id(),
      TRUE
    );
    $render = $this->renderer->renderPlain($postRender);

    // Set response.
    $response->addCommand(new ReplaceCommand($selector, $render));
    $response->setAttachments($postRender['#provider']['libraries']);

    // Set cookie.
    $cookie = new Cookie(ExternalMediaRender::SELECTOR_ITEM . '_' . $postRender['#provider']['id'], 'true');
    $response->headers->setCookie($cookie);

    return $response;
  }

  /**
   * Remove conscent cookie.
   *
   * @param \Drupal\media\Entity\Media $media
   *   The media entity.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   The ajax response.
   *
   * @throws \Exception
   */
  public function removeConscent($media): AjaxResponse {

    // Check if ajax request.
    if (!$this->request->isXmlHttpRequest()) {
      throw new NotFoundHttpException();
    }

    // Init ajax response.
    $response = new AjaxResponse();

    // Get html selector.
    $selector = '.' . ExternalMediaRender::SELECTOR_ITEM . '_' . $media->id();

    // Get html content.
    $postRender = $this->externalMediaRender->getRenderExternalMedia(
      $media->get('field_media_external_medias')->value,
      (int) $media->id(),
      FALSE,
      TRUE
    );
    $render = $this->renderer->renderPlain($postRender);

    // Set response.
    $response->addCommand(new ReplaceCommand($selector, $render));
    $response->setAttachments($postRender['#provider']['libraries']);

    // Remove cookie.
    $response->headers->clearCookie(ExternalMediaRender::SELECTOR_ITEM . '_' . $postRender['#provider']['id']);

    return $response;
  }

}
