<?php

declare(strict_types=1);

namespace Drupal\media_iframe_cookie\LazyBuilder;

use Drupal\Component\Utility\Html;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Security\TrustedCallbackInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Lazy builder external media render.
 */
class ExternalMediaRender implements TrustedCallbackInterface {

  use StringTranslationTrait;

  /**
   * The selector item.
   *
   * @var string
   */
  public const SELECTOR_ITEM = 'external_media';

  /**
   * The config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $config;

  /**
   * The request service.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * Provider.
   *
   * @var array
   */
  protected $provider;

  /**
   * Url of iframe.
   *
   * @var string
   */
  protected $url;

  /**
   * Media id.
   *
   * @var int
   */
  protected $mid;

  /**
   * Iframe array.
   *
   * @var array
   */
  protected $iframe;

  /**
   * Constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $string_translation
   *   The string translation service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $requestStack
   *   The request stack service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TranslationInterface $string_translation, RequestStack $requestStack) {
    $this->config = $config_factory->get('media_iframe_cookie.settings');
    $this->stringTranslation = $string_translation;
    $this->request = $requestStack->getCurrentRequest();
  }

  /**
   * {@inheritdoc}
   */
  public static function trustedCallbacks() {
    return ['getRenderExternalMedia'];
  }

  /**
   * Get render external media.
   *
   * @param string $html
   *   The iframe to trender.
   * @param int $mid
   *   The media id.
   * @param bool $applyPostRender
   *   Apply post render or not.
   * @param bool $removeConscent
   *   Remove conscent cookie.
   *
   * @return array
   *   The render array.
   */
  public function getRenderExternalMedia(string $html, int $mid, bool $applyPostRender = FALSE, bool $removeConscent = FALSE): array {
    $this->parseIframeHtml($html);

    // Set content data.
    $this->mid = $mid;
    $this->provider = $this->getProviderExternalMedia();

    // If provider is defined.
    if ($this->provider && !$removeConscent) {

      // Check cookie.
      $cookie = $this->request->cookies->get(self::SELECTOR_ITEM . '_' . $this->provider['id']);

      if ($cookie || $applyPostRender) {
        return $this->postRenderExternalMedia();
      }
    }

    return $this->preRenderExternalMedia();
  }

  /**
   * Parse iframe html.
   *
   * @param string $html
   *   The html to parse.
   */
  public function parseIframeHtml($html): void {

    // Get html code.
    $html = Html::load($html)->getElementsByTagName('iframe');

    // For each iframe tag.
    foreach ($html as $iframe) {
      $attributes = $iframe->attributes;

      foreach ($attributes as $attribute) {

        // Set iframe attributes.
        $this->iframe[$attribute->nodeName] = $attribute->value;

        // Set url.
        if ($attribute->nodeName === 'src') {
          $this->url = $attribute->value;
        }
      }
    }
  }

  /**
   * Get provider external media.
   *
   * @return array|null
   *   The render array or null
   */
  public function getProviderExternalMedia(): ?array {
    // Get provider for current url.
    foreach ($this->config->get('providers') as $key => $provider) {

      if (str_starts_with($this->url, $provider['domain'])) {
        $provider['id'] = $key;
        return $provider;
      }
    }

    return NULL;
  }

  /**
   * Get the pre render array for external media.
   *
   * @return array
   *   The render array.
   */
  protected function preRenderExternalMedia(): array {
    if ($this->provider) {
      return [
        '#theme' => 'external_media_prevalidate',
        '#class' => [
          self::SELECTOR_ITEM,
          self::SELECTOR_ITEM . '_' . $this->mid,
          'disabled',
        ],
        '#provider' => $this->provider,
        '#privacy' => [
          '#type' => 'link',
          '#title' => $this->t('Privacy terms'),
          '#url' => Url::fromUri($this->provider['privacy']),
          '#options' => [
            'attributes' => [
              'target' => '_blank',
              'aria-label' => $this->t('Privacy terms'),
              'title' => $this->t('Privacy terms'),
            ],
          ],
        ],
        '#validate' => [
          '#type' => 'link',
          '#title' => $this->t('Display content'),
          '#url' => Url::fromRoute('media_iframe_cookie.validate', [
            'media' => $this->mid,
          ]),
          '#options' => [
            'attributes' => [
              'class' => 'use-ajax',
              'aria-label' => $this->t('Display content'),
              'title' => $this->t('Display content'),
            ],
          ],
        ],
        '#attached' => [
          'library' => [
            'media_iframe_cookie/media_iframe_cookie',
          ],
        ],
      ];
    }

    return [
      '#markup' => $this->t('No provider found'),
    ];
  }

  /**
   * Get the final render array for external media.
   *
   * @return array
   *   The render array.
   */
  protected function postRenderExternalMedia(): array {
    return [
      '#theme' => 'external_media_validate',
      '#class' => [
        self::SELECTOR_ITEM,
        self::SELECTOR_ITEM . '_' . $this->mid,
        'enabled',
      ],
      '#provider' => $this->provider,
      '#iframe' => [
        '#type' => 'html_tag',
        '#tag' => 'iframe',
        '#attributes' => $this->iframe,
      ],
      '#unvalidate' => [
        '#type' => 'link',
        '#title' => $this->t('Remove conscent'),
        '#url' => Url::fromRoute('media_iframe_cookie.remove_conscent', [
          'media' => $this->mid,
        ]),
        '#options' => [
          'attributes' => [
            'class' => 'use-ajax',
            'aria-label' => $this->t('Remove conscent'),
            'title' => $this->t('Remove conscent'),
          ],
        ],
      ],
      '#attached' => [
        'library' => [
          'media_iframe_cookie/media_iframe_cookie',
        ],
      ],
    ];
  }

}
